package teamcity.crowd.integration;

import com.atlassian.crowd.model.user.User;
import jetbrains.buildServer.controllers.interceptors.auth.HttpAuthenticationResult;
import jetbrains.buildServer.controllers.interceptors.auth.HttpAuthenticationScheme;
import jetbrains.buildServer.serverSide.auth.ServerPrincipal;
import org.jetbrains.annotations.NotNull;
import teamcity.crowd.integration.utils.Maybe;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

public class CrowdAuthenticationScheme implements HttpAuthenticationScheme {
    public static final String REALM = "crowd";

    private CrowdIntegrationClient crowdIntegrationClient;

    public CrowdAuthenticationScheme(CrowdIntegrationClient crowdIntegrationClient) {
        this.crowdIntegrationClient = crowdIntegrationClient;
    }

    @NotNull
    @Override
    public HttpAuthenticationResult processAuthenticationRequest(@NotNull HttpServletRequest httpServletRequest, @NotNull HttpServletResponse httpServletResponse) throws IOException {
        Maybe<User> possibleUser = crowdIntegrationClient.userAlreadyLoggedIntoCrowd(httpServletRequest);
        if (possibleUser.hasValue()){
            User user = possibleUser.getValue();
            return HttpAuthenticationResult.authenticated(new ServerPrincipal(REALM, user.getName()));
        }
        return HttpAuthenticationResult.notApplicable();
    }

}
