package teamcity.crowd.integration;

import com.atlassian.crowd.exception.*;
import com.atlassian.crowd.integration.http.CrowdHttpAuthenticator;
import com.atlassian.crowd.model.user.User;
import com.atlassian.crowd.service.client.CrowdClient;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.springframework.beans.factory.annotation.Autowired;
import teamcity.crowd.integration.utils.Maybe;
import teamcity.crowd.integration.utils.Nothing;
import teamcity.crowd.integration.utils.Something;

import javax.servlet.http.HttpServletRequest;

public class RealCrowdIntegrationClient implements CrowdIntegrationClient {
    private Log log = LogFactory.getLog(CrowdIntegrationClient.class);

    private CrowdHttpAuthenticator crowdHttpAuthenticator;

    private CrowdClient crowdClient;

    public RealCrowdIntegrationClient(CrowdHttpAuthenticator crowdHttpAuthenticator, CrowdClient crowdClient) {
        this.crowdHttpAuthenticator = crowdHttpAuthenticator;
        this.crowdClient = crowdClient;
    }

    @Override
    public Maybe<User> loginUserWithPassword(String username, String password) {
        try {
            User user = crowdClient.authenticateUser(username, password);
            return new Something<User>(user);
        } catch (UserNotFoundException e) {
            e.printStackTrace();
        } catch (InactiveAccountException e) {
            e.printStackTrace();
        } catch (ExpiredCredentialException e) {
            e.printStackTrace();
        } catch (ApplicationPermissionException e) {
            e.printStackTrace();
        } catch (InvalidAuthenticationException e) {
            e.printStackTrace();
        } catch (OperationFailedException e) {
            e.printStackTrace();
        }
        return new Nothing<User>();
    }

    @Override
    public Maybe<User> userAlreadyLoggedIntoCrowd(HttpServletRequest request) {

        try {
            User user = crowdHttpAuthenticator.getUser(request);
            log.warn("User is " + user);
            return user != null ? new Something<User>(user) : new Nothing<User>();
        } catch (InvalidTokenException e) {
            e.printStackTrace();
        } catch (ApplicationPermissionException e) {
            e.printStackTrace();
        } catch (InvalidAuthenticationException e) {
            e.printStackTrace();
        } catch (OperationFailedException e) {
            e.printStackTrace();
        }

        return new Nothing<User>();
    }

}
