package teamcity.crowd.integration.loginModule;

import com.atlassian.crowd.model.user.User;
import jetbrains.buildServer.log.Loggers;
import jetbrains.buildServer.serverSide.auth.ServerPrincipal;
import teamcity.crowd.integration.CrowdAuthenticationScheme;
import teamcity.crowd.integration.CrowdIntegrationClient;
import teamcity.crowd.integration.utils.Maybe;

import javax.security.auth.Subject;
import javax.security.auth.callback.Callback;
import javax.security.auth.callback.CallbackHandler;
import javax.security.auth.callback.NameCallback;
import javax.security.auth.callback.PasswordCallback;
import javax.security.auth.login.FailedLoginException;
import javax.security.auth.login.LoginException;
import javax.security.auth.spi.LoginModule;
import java.util.Map;

public class CrowdLoginModule implements LoginModule {

    private CallbackHandler myCallbackHandler;
    private NameCallback myNameCallback;
    private PasswordCallback myPasswordCallback;
    private Callback[] myCallbacks;
    private Subject mySubject;

    private CrowdIntegrationClient crowdIntegrationClient;

    @Override
    public void initialize(Subject subject, CallbackHandler callbackHandler, Map<String, ?> sharedState, Map<String, ?> options) {
        myCallbackHandler = callbackHandler;
        myNameCallback = new NameCallback("login:");
        myPasswordCallback = new PasswordCallback("password:", false);
        myCallbacks = new Callback[]{myNameCallback, myPasswordCallback};
        mySubject = subject;
        crowdIntegrationClient = (CrowdIntegrationClient) options.get(CrowdIntegrationClient.class.getName());
    }

    @Override
    public boolean login() throws LoginException {
        try {
            myCallbackHandler.handle(myCallbacks);
        } catch (Throwable t) {
            throw new LoginException(t.toString());
        }
        final String username = myNameCallback.getName();
        final String password = new String(myPasswordCallback.getPassword());

        String message = String.format("Attempting to log in with user [%s] and password [%s]", username, password);
        Loggers.SERVER.warn(message);

        Maybe<User> possiblyLoggedInUser = crowdIntegrationClient.loginUserWithPassword(username, password);
        if (possiblyLoggedInUser.hasValue()) {
            mySubject.getPrincipals().add(new ServerPrincipal(CrowdAuthenticationScheme.REALM, possiblyLoggedInUser.getValue().getName()));
            return true;
        }
        throw new FailedLoginException("Invalid username or password");
    }

    @Override
    public boolean commit() throws LoginException {
        return true;
    }

    @Override
    public boolean abort() throws LoginException {
        return true;
    }

    @Override
    public boolean logout() throws LoginException {
        return true;
    }
}
