package teamcity.crowd.integration.loginModule;

import jetbrains.buildServer.serverSide.auth.LoginConfiguration;
import jetbrains.buildServer.serverSide.auth.LoginModuleDescriptor;
import org.jetbrains.annotations.Nullable;
import teamcity.crowd.integration.CrowdIntegrationClient;
import teamcity.crowd.integration.sync.SyncInitializer;

import javax.security.auth.spi.LoginModule;
import java.util.HashMap;
import java.util.Map;

public class CrowdLoginModuleDescriptor implements LoginModuleDescriptor {

    private final CrowdIntegrationClient crowdIntegrationClient;
    private SyncInitializer syncInitializer;

    public CrowdLoginModuleDescriptor(LoginConfiguration loginConfiguration, CrowdIntegrationClient crowdIntegrationClient,
                                      SyncInitializer syncInitializer){
        this.crowdIntegrationClient = crowdIntegrationClient;
        this.syncInitializer = syncInitializer;
        loginConfiguration.registerLoginModule(this);
        initializeCrowdSync();
    }

    private void initializeCrowdSync() {
        syncInitializer.initialize();
    }

    @Override
    public Class<? extends LoginModule> getLoginModuleClass() {
        return CrowdLoginModule.class;
    }

    @Nullable
    @Override
    public Map<String, ?> getOptions() {
        Map<String, Object> options = new HashMap<String, Object>();
        options.put(CrowdIntegrationClient.class.getName(), crowdIntegrationClient);
        return options;
    }

    @Nullable
    @Override
    public String getTextForLoginPage() {
        return "So you think you can log in, huh? Ok. Gimme the user and password! Now!";
    }

    @Nullable
    @Override
    public String getDisplayName() {
        return "Crowd Awesome authentication plugin";
    }
}
