package teamcity.crowd.integration.sync;

import jetbrains.buildServer.util.Alarm;
import teamcity.crowd.integration.utils.LoggerProvider;
import teamcity.crowd.integration.utils.PluginConfiguration;
import teamcity.crowd.integration.utils.TaskScheduler;

public class CrowdSyncInitializer implements SyncInitializer {
    private CrowdSync crowdGroupsSync;
    private CrowdSync crowdUsersSync;
    private LoggerProvider loggerProvider;
    private PluginConfiguration pluginConfiguration;
    private TaskScheduler taskScheduler;
    private int ONE_SECOND = 1000;

    public CrowdSyncInitializer(CrowdSync crowdGroupsSync, CrowdSync crowdUsersSync, LoggerProvider loggerProvider,
                                PluginConfiguration pluginConfiguration, TaskScheduler taskScheduler) {
        this.crowdGroupsSync = crowdGroupsSync;
        this.crowdUsersSync = crowdUsersSync;
        this.loggerProvider = loggerProvider;
        this.pluginConfiguration = pluginConfiguration;
        this.taskScheduler = taskScheduler;
    }

    @Override
    public void initialize() {
        if (pluginConfiguration.isTeamCityGroupsSyncEnabled()) {
            initializeGroupSync();
        }
        if (pluginConfiguration.isTeamCityUsersSyncEnabled()) {
            initializeUserSync();
        }
    }

    private void initializeUserSync() {
        final int syncEvery = pluginConfiguration.getTeamCityUsersSyncInterval();
        final int syncDelay = pluginConfiguration.getTeamCitySyncDelay();

        taskScheduler.scheduleTask(crowdUsersSync, "SyncCrowdUsers", ONE_SECOND * syncDelay, ONE_SECOND * syncEvery);
        loggerProvider.getServerLogger().info(
                String.format("========== Scheduled Crowd Users Sync to run every [%d] seconds =========", syncEvery));

    }

    private void initializeGroupSync() {
        final int syncEvery = pluginConfiguration.getTeamCityGroupsSyncInterval();
        final int syncDelay = pluginConfiguration.getTeamCitySyncDelay();

        taskScheduler.scheduleTask(crowdGroupsSync, "SyncCrowdGroups", ONE_SECOND * syncDelay, ONE_SECOND * syncEvery);
        loggerProvider.getServerLogger().info(
                String.format("========== Scheduled Crowd Groups Sync to run every [%d] seconds =========", syncEvery));
    }


}
