package teamcity.crowd.integration.utils;

import com.atlassian.crowd.service.client.ClientProperties;
import jetbrains.buildServer.web.openapi.PluginException;
import org.springframework.beans.factory.annotation.Value;

import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.Properties;

public class PluginConfiguration {
    private int teamCityGroupsSyncInterval = 60;

    private int teamCityUsersSyncInterval = 300;

    private boolean teamCityUsersSyncEnabled = true;

    private boolean teamCityGroupsSyncEnabled = true;

    private int teamCitySyncDelay = 120;

    private LoggerProvider loggerProvider;
    private String configDir;
    private String configurationFileName;
    private ClientProperties clientProperties;

    public PluginConfiguration() {
    }

    public PluginConfiguration(String configDir, String configurationFileName, ClientProperties clientProperties, LoggerProvider loggerProvider) {
        this.configDir = configDir;
        this.configurationFileName = configurationFileName;
        this.clientProperties = clientProperties;
        this.loggerProvider = loggerProvider;

        initPropertiesFromFile();
    }

    public int getTeamCityGroupsSyncInterval() {
        return teamCityGroupsSyncInterval;
    }

    public int getTeamCityUsersSyncInterval() {
        return teamCityUsersSyncInterval;
    }

    public int getTeamCitySyncDelay() {
        return teamCitySyncDelay;
    }

    public boolean isTeamCityUsersSyncEnabled() {
        return teamCityUsersSyncEnabled;
    }

    public boolean isTeamCityGroupsSyncEnabled() {
        return teamCityGroupsSyncEnabled;
    }

    public ClientProperties getClientProperties() {
        return clientProperties;
    }

    private void initPropertiesFromFile() {
        final File configurationFile = new File(configDir, configurationFileName);
        final Properties teamCityPluginProperties = new Properties();
        try {
            teamCityPluginProperties.load(new FileReader(configurationFile));

//            logPropertiesFromFile(teamCityPluginProperties);

            nowSetPropertiesThatAreInConfigFile(teamCityPluginProperties);
        } catch (IOException e) {
            loggerProvider.getServerLogger().error("A very odd situation. The file reports as Exists but cannot be found.", e);
            throw new PluginException("Can't initialize the configuration properties for TeamCity Crowd Integration plugin. Make sure configuration file teamcity-crowd-plugin.properties is in the TeamCity Data Folder/config", e);
        }
    }

    private void nowSetPropertiesThatAreInConfigFile(Properties teamCityPluginProperties) {
        if (teamCityPluginProperties.containsKey("tc.groups.sync.interval")) {
            teamCityGroupsSyncInterval = Integer.parseInt(teamCityPluginProperties.getProperty("tc.groups.sync.interval"));
        }
        if (teamCityPluginProperties.containsKey("tc.users.sync.interval")) {
            teamCityUsersSyncInterval = Integer.parseInt(teamCityPluginProperties.getProperty("tc.users.sync.interval"));
        }
        if (teamCityPluginProperties.containsKey("tc.sync.delay")) {
            teamCitySyncDelay = Integer.parseInt(teamCityPluginProperties.getProperty("tc.sync.delay"));
        }
        if (teamCityPluginProperties.containsKey("tc.users.sync.enabled")) {
            teamCityUsersSyncEnabled = Boolean.parseBoolean(teamCityPluginProperties.getProperty("tc.users.sync.enabled"));
        }
        if (teamCityPluginProperties.containsKey("tc.groups.sync.enabled")) {
            teamCityGroupsSyncEnabled = Boolean.parseBoolean(teamCityPluginProperties.getProperty("tc.groups.sync.enabled"));
        }
        clientProperties.updateProperties(teamCityPluginProperties);

//        logUpdatedClientProperties(clientProperties);
    }

//    private void logUpdatedClientProperties(ClientProperties clientProperties) {
//        StringBuilder builder = new StringBuilder("------------------------------ Updated Client Properties ----------------------------------");
//        builder.append("[application.name] = ").append(clientProperties.getApplicationName());
//        builder.append("[application.password] = ").append(clientProperties.getApplicationPassword());
//        builder.append("[application.login.url] = ").append(clientProperties.getApplicationAuthenticationURL());
//        builder.append("[crowd.base.url] = ").append(clientProperties.getBaseURL());
//        builder.append("------------------------------ ----------------------------- ----------------------------------");
//        loggerProvider.getServerLogger().info(builder.toString());
//
//    }

//    private void logPropertiesFromFile(Properties teamCityPluginProperties) {
//        final StringBuilder builder = new StringBuilder("------- Instantiating TeamCity Crowd Integration plugin with properties ------- ");
//        for (String propertyKey : teamCityPluginProperties.stringPropertyNames()) {
//            builder.append("[").append(propertyKey).append("] = ").append(teamCityPluginProperties.getProperty(propertyKey)).append("\n");
//        }
//        builder.append("------------------------------------------------------------------------------------");
//        loggerProvider.getServerLogger().info(builder.toString());
//    }
}
