package teamcity.crowd.integration.sync

import teamcity.crowd.integration.mocks.MockLogger
import teamcity.crowd.integration.utils.PluginConfiguration
import teamcity.crowd.integration.utils.TaskScheduler

class CrowdSyncInitializerTest extends GroovyTestCase {
    private def loggerProvider = new MockLogger()
    private def crowdGroupsSync = [:] as CrowdSync
    private def crowdUsersSync = [:] as CrowdSync

    void test_shouldScheduleGroupsSync() {
        def pluginConfiguration = [
                isTeamCityUsersSyncEnabled: { false },
                isTeamCityGroupsSyncEnabled: { true },
                getTeamCityGroupsSyncInterval: { 64 },
                getTeamCitySyncDelay: { 22 }
        ] as PluginConfiguration
        def taskScheduler = [scheduleTask:
                { runnable, taskName, delay, interval ->
                    assert runnable == crowdGroupsSync
                    assert taskName == "SyncCrowdGroups"
                    assert delay == 1000 * 22
                    assert interval == 1000 * 64
                }
        ] as TaskScheduler

        def initializer = new CrowdSyncInitializer(crowdGroupsSync, crowdUsersSync, loggerProvider, pluginConfiguration, taskScheduler)
        initializer.initialize()
    }

    void test_shouldScheduleUsersSync() {
        def pluginConfiguration = [
                isTeamCityUsersSyncEnabled: { true },
                isTeamCityGroupsSyncEnabled: { false },
                getTeamCityUsersSyncInterval: { 46 },
                getTeamCitySyncDelay: { 21 }
        ] as PluginConfiguration
        def taskScheduler = [scheduleTask:
                { runnable, taskName, delay, interval ->
                    assert runnable == crowdUsersSync
                    assert taskName == "SyncCrowdUsers"
                    assert delay == 1000 * 21
                    assert interval == 1000 * 46
                }
        ] as TaskScheduler

        def initializer = new CrowdSyncInitializer(crowdGroupsSync, crowdUsersSync, loggerProvider, pluginConfiguration, taskScheduler)
        initializer.initialize()
    }
}
